package de.devbliss.paulparser;

import java.util.Date;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * A container class that constructs a clean and nice PublicationModel from a
 * PublicationModelRaw.
 *
 * More information right at the constructor.
 *
 * In general we follow:
 * http://scholar.google.com/intl/en/scholar/inclusion.html
 */
public class PublicationModel {

    public Set<String> authors;
    public String title;

    public Date date;
    // when this paper is a journal:
    public String journalTitle;
    public String journalIssue;
    public String journalVolume;
    public String journalPages;
    // Leave this as a plain String, don't parse, because for many journals, the date is meant to be a String, eg,
    // Spring 2010
    public String journalDate;

    public String doi;

    public String description;


    /**
     * PublicationModelRaw usually comes from the PublicationMetaTagParser and contains
     * a lot of duplicate and raw information.
     * <p/>
     * If you construct the PublicationModel via the constructor all
     * data of PublicationModelRaw is integrated in a clean way and de-duplicated where
     * possible.
     * <p/>
     * You end up having a "nice" and "clean" usable object of the publication metadata.
     * <p/>
     * In general we follow:
     * http://scholar.google.com/intl/en/scholar/inclusion.html
     *
     * @param model
     */
    public PublicationModel(PublicationModelRaw model) {

        authors = new LinkedHashSet<String>();

        // extract authors from raw model:
        doIntegrateAuthors(model);

        // extract title from raw model:
        doIntegrateTitle(model);

        doIntegrateSource(model);

        doIntegrateDoi(model);

        doIntegrateDate(model);

        doIntegrateDescription(model);
    }

    /**
     * not sure why you want to use that... except you are a unit test...
     */
    public PublicationModel() {
        authors = new HashSet<String>();
    }

    private void doIntegrateAuthors(PublicationModelRaw model) {

        // first check citation_
        if (model.citation_author.size() > 0) {

            for (String author : model.citation_author) {
                authors.add(author);
            }

        // then check dc creators:
        } else if (model.dcCreator.size() > 0) {

            for (String author : model.dcCreator) {
                authors.add(author);
            }

        // citation_authors is usually a line with ";" as separator like so
        // "first author; second author; third author"
        } else if (model.citation_authors != null) {

            // split them. All journals we know of use ";" as separator
            String[] authorSplit = model.citation_authors
                    .split(";");
            if (authorSplit.length > 0) {
                for (String author : authorSplit) {
                    //add and dont't forget to trim :)
                    authors.add(author.trim());
                }
            }

        // then check contributors (that might be wrong, but we accept it
        // anyways as some journals are using that.)
        } else if (model.dcContributor.size() > 0) {

            for (String author : model.dcContributor) {
                authors.add(author);
            }
        }
    }

    private void doIntegrateTitle(PublicationModelRaw model) {
        title = selectFirst(model.citation_title, model.dcTitle);
    }

    private void doIntegrateDate(PublicationModelRaw model) {
        date = selectFirstDate(model.citation_date, model.dcDate,
                model.dcIssued, model.citation_publication_date,
                model.prismPublicationDate);
    }

    private void doIntegrateSource(PublicationModelRaw model) {
        journalTitle = selectFirst(model.citation_journal_title, model.prismPublicationName);
        journalIssue = selectFirst(model.citation_issue, model.prismIssue);
        journalVolume = selectFirst(model.citation_volume, model.prismVolume);
        journalPages = selectFirst(model.citation_firstpage, model.prismStartingPage);
        if (journalPages != null) {
            String lastPage = selectFirst(model.citation_lastpage, model.prismEndingPage);
            if (lastPage != null) {
                journalPages += "-" + lastPage;
            }
        }
        journalDate = selectFirst(model.citation_publication_date, model.prismPublicationDate);
    }

    private void doIntegrateDoi(PublicationModelRaw model) {

        String tempDoi = null;

        if (model.citation_doi != null) {
            tempDoi = model.citation_doi;

            // Note: A "dc.identifier" may not necessarily be a doi.
            // But in our scientific community it is in almost all cases we know of.
        } else if (model.dcIdentifier != null) {
            tempDoi = model.dcIdentifier;
        }

        // Don't forget to clean doi...
        // ...sometimes dois start with "doi:"
        // that is fine, but by convention we remove it.
        if (tempDoi != null) {
            if (tempDoi.startsWith("doi:")) {
                tempDoi = tempDoi.substring(4, tempDoi.length());
            }
            doi = tempDoi;
        }
    }

    private void doIntegrateDescription(PublicationModelRaw model) {
        if (model.dcDescription != null) {
            description = model.dcDescription;
        }
    }

    private String selectFirst(String... values) {
        for (String value : values) {
            if (value != null) {
                return value;
            }
        }
        return null;
    }

    private Date selectFirstDate(String... values) {
        for (String value : values) {
            if (value != null) {
                Date date = FuzzyDateParser.parse(value);
                if (date != null) {
                    return date;
                }
            }
        }
        return null;
    }

    /**
     * Convenience method that follows guidelines from
     * http://scholar.google.com/intl/en/scholar/inclusion.html
     * <p/>
     * Therefore we say "yes" when at least the following criteria can be
     * deducted:
     * - authors
     * - title
     * - publication_date
     */
    boolean isValidCitation() {
        return title != null && journalTitle != null && authors.size() > 0;
    }
}
